package POE::Component::Leve::visualShell;

use warnings;
use strict;
use Carp;

##############################################################################
=pod

=head1 NAME

visualShell

=head1 DESCRIPTION

puts up a terminal interface to the leve system

=head1 AUTHOR

  Bruno Bttcher <bboett at adlp dot org>

=head1 COPYRIGHT and LICENCE

 Copyright (c) 2002 Bruno Boettcher

 visualShell.pl is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


=cut

##############################################################################
#sub POE::Kernel::ASSERT_EVENTS () { 1 }

use POE;
#use POE qw/Session/;
use vars qw/$VERSION/;
#use Term::ReadLine;
#use POE::Wheel::ReadLine;
use Term::Visual;
#use POE::Wheel::Run;
use POE::Filter::Reference;
use Data::Dumper;

$VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);

######################################################################
=pod

=item new 

constructor for the new leve shell

=cut

######################################################################
sub new 
{
  my $fs = shift;
  my $serverName = shift;
  my $o_xchatHandler = { };

  bless $o_xchatHandler, "POE::Component::Leve::visualShell";
  return $o_xchatHandler;
}# sub new 
##############################################################################
=pod

=head1 Methods of this class

=over

=item init

set up the main datastructures of this object, make it ready for the run

=cut

######################################################################
sub init
{
  my($this,$settings) = @_;

  $this->{"settings"} ={};
  $this->{"completions"} = [];
  $this->{"settings"} = $settings if($settings);
  $this->{"prompt"} = "cmd> ";
  $this->{"name"}  = "Leveloda shell";

  $this->instantiateTerm(@_);

  POE::Session->create( 
      #Alias => "visualShell",
      object_states => 
      [

      $this =>
      {  _start => "_start",
	_stop  => "_stop",
	got_term_input => "got_input_handler",
	#update_time    => "update_time",
	#update_name    => "update_name",
	#test_stderr    => "test_stderr",
	task_result => "handle_task_result",
	task_done   => "handle_task_done",
	task_debug  => "handle_task_debug",
	quit  => "quit",
      },

  ]
    );
  #$poe_kernel->run();
  print("exiting shell init\n");
}#sub init
##############################################################################
=pod

=item start of the poe session


=cut

######################################################################
sub _start 
{
  my ($this, $kernel, $heap, $dsn, $queries,$session) = @_[OBJECT, KERNEL, HEAP, ARG0, ARG1,SESSION];
  #$this->Debug("_start  args : ".Dumper(\@_)."\n"); 
  $kernel->alias_set("visualShell");
  $this->{"kernel"} = $kernel;
  $this->{"heap"} = $heap;
  #$session->option(default => 1);
  #print("_start::SH starting shell\n");
  #$kernel->post('visualShell' => "runShell");
  #$kernel->post('runShell');
  $heap->{"visualShell"} = $this;

  $this->runShell(@_);
  print("success starting shell!!\n");

  # Tell the terminal to send me input as "got_term_input".
  $kernel->post( interface => send_me_input => "got_term_input" );
  #$this->Debug("send data grab event\n");

  # Start updating the time.
  #$kernel->yield( "update_time" );
  $kernel->yield( "update_name" );
  $kernel->yield( "test_stderr" );
  #$this->Debug("Exit of the while loop, program end reached\n");
  $this->Debug("start for shell finished\n");
  my $temp = 0; Curses::mousemask(0, $temp);


}#sub _start
##############################################################################
=pod

=item stop of the poe session


=cut

######################################################################
sub _stop 
{
  my ($this, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
  $this->{"heap"} = $heap;
  $this->{"kernel"} = $kernel;
  $this->{"term"}->delete_window($this->{"termid"});
  $kernel->post('debug',"stopped series manager");

}#sub _stop 
##############################################################################
=pod

=item default of the poe session


=cut

######################################################################
sub _default 
{
  my ($this, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
  $this->{"heap"} = $heap;
  $this->{"kernel"} = $kernel;
  #print("defaultped ".$_[ARG0]."\n");
  $this->Debug("default ".$_[ARG0]."\n");
  # for (my $i=0; $i < 8 ; $i++)
  #{
    #  my $arg = $_[eval (print("ARG$i"))];
    #  print(" $i= $arg") if($arg); 
    #}# for (my $i=0; $i < 8 ; $i++)
  $this->Debug(" 1= ".Dumper($_[ARG1])) if($_[ARG1]); 
  $this->Debug(" 2= ".Dumper($_[ARG2])) if($_[ARG2]); 
  $this->Debug(" 3= ".Dumper($_[ARG3])) if($_[ARG3]); 
  $this->Debug(" 4= ".Dumper($_[ARG4])) if($_[ARG4]); 
  $this->Debug(" 5= ".Dumper($_[ARG5])) if($_[ARG5]); 
  $this->Debug(" 6= ".Dumper($_[ARG6])) if($_[ARG6]); 
  $this->Debug(" 7= ".Dumper($_[ARG7])) if($_[ARG7]); 
  $this->Debug(" 8= ".Dumper($_[ARG8])) if($_[ARG8]); 
  $this->Debug("\n");
}#sub _default 
##############################################################################
=pod

=item instantiateTerm

create the shell object ready to start the session after that


=cut

######################################################################
sub instantiateTerm
{
  my ($this) = @_;
  if(!$this) 
  {
    die("ayaaahhh where went my own ref??? \n");
  }
  #
  #$this->Debug("help! we have no kernel defined!!!\n") if(!$kernel);
  my $prompt = $this->{"prompt"};
  my $shCommands = do("shellCmds.pm");
  #add internal commands
  my $localCmds = {
    "shutdown" => "shutdown",
    "quit" => "quit",
    "resynch" => "none",
  };
  #easier for the autocompletion to add them to the global list...
  $shCommands->{"shutdown"} = "shutdown";
  $shCommands->{"quit"} = "quit";
  $shCommands->{"resynch"} = "none";

  #make global
  $this->{"shellCmds"} = $shCommands;
  $this->{"localCmds"} = $localCmds;


  my $vt = Term::Visual->new( Alias => "interface",
      Errlevel => 0 );

#set this element as the callback
  $vt->callBack($this);

# $vt->set_palette( mycolor       => "magenta on black",
#	    statcolor     => "green on black",
#	    sockcolor     => "cyan on black",
#	    ncolor        => "white on black",
#	    st_frames     => "bright cyan on blue",
#	    st_values     => "bright white on blue",
#	    stderr_bullet => "bright white on red",
#	    stderr_text   => "bright yellow on black",
#	    err_input     => "bright white on red",
#	    help          => "white on black",
#	    help_cmd      => "bright white on black" );

  $vt->set_palette(  ncolor => "green on black");
  $vt->set_palette(  mycolor => "bright blue on black");

  my $window_id = $vt->create_window(
      Window_Name => "window_one",

      Status => { 0 =>
      { format => "\0(st_frames)" .
      " [" .
      "\0(st_values)" .
    "%8.8s" .
    "\0(st_frames)" .
    "] " .
    "\0(st_values)" .
    "%s",
  fields => [qw( time name )] },
      },

      Buffer_Size => 1000,
      History_Size => 50,
      Use_Status   => 0,
      Use_Title    => 0,

      #Title => "Title of window_one"  
      );

      $this->{"term"} = $vt;
      $this->{"termid"} = $window_id;
      #instatianted the thing...
}#sub instantiateTerm
##############################################################################
=pod

=item runShell

start and run the shell, this is an interactive process, so it won't exit
unlees the user asks for it! this blocks all if not called from the correct
spot!


=cut

######################################################################
sub runShell
{
  my ($this) = $_[OBJECT];
  #$this->Debug("Dumping args : ".Dumper(\@_)."\n");
  my $heap = $this->{"heap"};
  my $kernel = $this->{"kernel"};
  $kernel = $_[KERNEL] if(!$kernel);
  $heap = $_[HEAP] if(!$heap);

  if(!$this) 
  {
    $this = $heap->{"visualShell"} ;
    #$this->Debug("fetchedt ref to object from heap....\n");
  }
  #$this->Debug("help! we have no kernel defined!!!\n") if(!$kernel);


  my $vt = $this->{"term"};
  my $window_id = $this->{"termid"};
  $vt->print($window_id, "My Window ID is $window_id");
  #starting the thing...
}#sub runShell
######################################################################
=pod

=item quit 

shutdown the whole thing

=cut

######################################################################
sub quit
{
  #my ($this,$msg,$kernel,$heap) = @_;
  my ($this,$kernel, $heap,$msg) = @_[OBJECT,KERNEL, HEAP,ARG0];

  $this->Debug("$msg\n");
  $heap->{wheel}->clear();
  $kernel->yield("shutdown");
}#sub quit
######################################################################
=pod

=item delegate 

send the stuff piped here to the controlling session

=cut

######################################################################
sub delegate
{
  my ($this, $kernel, $heap, $dsn, $queries) = @_[OBJECT, KERNEL, HEAP, ARG0, ARG1];

  my @request = ($_);
  $this->Debug("put $_ in array sending all to server\n");
  $this->{"server"}->put( \@request );
}#sub delegate
######################################################################
=pod

=item handle_task_result 

Handle information returned from the task.  Since we're using
POE::Filter::Reference, the $result is however it was created in the
child process.  In this sample, it's a hash reference.

=cut

######################################################################
sub handle_task_result 
{
  my ($this,$result) = @_[OBJECT,ARG0];
  $this->Debug ("Result for $result->{task}: $result->{status}\n");
}
######################################################################
=pod

=item handle_task_debug 

Catch and display information from the child's STDERR.  This was
useful for debugging since the child's warnings and errors were not
being displayed otherwise.

=cut

######################################################################
sub handle_task_debug 
{
  my ($this,$result) = @_[OBJECT,ARG0];
  $this->Debug ("$this->Debug: $result\n");
}
######################################################################
=pod

=item handle_task_done 

The task is done.  Delete the child wheel, and try to start a new
task to take its place.

=cut

######################################################################
sub handle_task_done 
{
  my ( $kernel, $heap, $task_id ) = @_[ KERNEL, HEAP, ARG0 ];
  delete $heap->{task}->{$task_id};
}
######################################################################
=pod

=item Debug 

an alternative to the print method prints out on the console but as the
same time pastes the stuff also into a log file

=cut

######################################################################
sub Debug
{
  my ($this,$msg) = @_;
  if(open(DIRLIST,">>debug"))
  {
    #print $msg;
    if($this->{"term"})
    {

      $this->print( $msg);
    }# if($this->{"term"})
    else
    {
      print("ARGH:$msg");
    }# else

    print DIRLIST $msg;
    close( DIRLIST );
  }# if(open(DIRLIST,">>debug"))
}#sub $this->Debug
######################################################################
=pod

=item print 

wrapper for the print method, prints to a visual instance if present otherwise to the console

=cut

######################################################################
sub print
{
  my ($this,$msg) = @_;

  if($this->{"term"})
  {
    my @lines = split(/\n/,$msg);
    foreach my $line (@lines)
    {
      chomp($line);
      $this->{"term"}->print($this->{"termid"}, "$line \n") if(!($line =~ /^\s*$/));
    }# foreach my $line (@lines)
  }
  else
  {
    print("NOSHEL: ". $msg);
  }
}# sub print
######################################################################
=pod

=item server 

Getter for the server instance the thing that allows us to communicate
with the patternServer

=cut

######################################################################
sub server
{
  my($this,$server) = @_;

  if($server)
  {
    $this->{"server"} = $server;
  }# if(open(DIRLIST,">>debug"))
  else
  {
    $server = $this->{"server"};
  }# else
  return $server;
}# sub setServer
######################################################################
=pod

=item got_input_handler 

some input went in from the visaul instance, handle it

=cut

######################################################################

sub got_input_handler
{
  my ($this,$heap, $input, $exception,$kernel) = @_[OBJECT,HEAP, ARG0, ARG1,KERNEL];

  my $shCommands = $this->{"shellCmds"};
  #$this->Debug( "entering got_input_handler with $input\n");


  if (defined $input)
  {
    my $prompt = "\0(mycolor)".$this->{"prompt"};
    #$heap->{wheel}->addhistory($input);

    chomp($input);

    #$this->Debug("incoming line = $input\n");
    #$heap->{wheel}->put("\tGot: $input");
    #echoaoa
    #my %palette = $this->{"term"}->get_palette();
    #NOTE: (ncolor, st_values, st_frames, stderr_text, stderr_bullet, statcolor)
    #                  are set and used by Term::Visual internally.
    #mycolor thiscolor
    #$this->{"term"}->set_palette(thiscolor        => "green on black"  );
    #my ($cname,$color_desc) = $this->{"term"}->get_palette("ncolor","yes?");
    #$this->Debug("actual color = $cname ".Dumper($color_desc)."\n");
    #$this->{"term"}->set_palette(  ncolor => "bold on black"  );
    #$this->{"term"}->set_palette(  ncolor => "green on black"  );
    #my ($cname2,$dumm) = $this->{"term"}->get_palette("ncolor","yes?");
    #$this->Debug("new color = $cname2 ".Dumper($color_desc)."\n");
    $this->{"term"}->print($this->{"termid"}, $prompt.$input);
    #$this->{"term"}->set_palette( ncolor        => $cname );
    #$this->{"term"}->set_palette( %palette );


    if($input =~ /^\s*shutdown/i)
    {
      $this->Debug("case shutdown\n");
      #$running = 0;
      my @request = ( "shutdown" );
      $this->{"server"}->put( \@request );

      my $msg = "Shutting down the server and quitting the processing";
      $kernel->delay_add( "quit", 1, $msg);
      #$this->quit("Shutting down the server and quitting the processing",$kernel,$heap);
    }# if(/shutdown/i)
    elsif($input =~ /^\s*resynch/i)
    {
      my @request = ( "rawnames" );
      $this->{"server"}->put( \@request );
    }# if(/shutdown/i)
    elsif($input =~ /^\s*quit/i)
    {
      #$this->Debug("case quit\n");
      #$running = 0;
      #$this->quit("exiting the client",$kernel,$heap);
      my $msg = "exiting the client";
      $kernel->delay_add( "quit", 0, $msg);
    }# if(/shutdown/i)
    elsif($input =~ /^\s*help/i)
    {
      my @names = keys(%{$this->{"localCmds"}});
      #TODO check for series and wish and resynch
      $this->Debug("Local cmds: ".join(' ',@names)."\n");
      if($kernel)
      {
	my @request = ($input);
	$this->{"server"}->put( \@request );
      }# if($kernel)
    }#elsif($input =~ /help/i)
    else
    {
      #$this->Debug("case otherwise\n");
      #my @request = ($_);
      #print("put $_ in array sending all to server\n");
      #$this->{"server"}->put( \@request );
      if($kernel)
      {
	#$this->Debug("sending $input to the server\n");
	#$kernel->post('delegate',$input);
	my @request = ($input);
	$this->{"server"}->put( \@request );
	#$this->Debug("posted $input to server\n");
      }# if($kernel)
      else
      {
	$this->Debug( "can't call delegate $input since we have no kernel defined!!\n");
      }#else

    }# else
    #$this->prompt(); # get another line
  }# if (defined $input)
  else 
  {
    #$heap->{wheel}->put("\tException: $exception");
    warn("\tException: $exception");
  }
}#sub got_input_handler
######################################################################
=pod

=item keypressed 

some key was pressed in the visaul console, lookit up, if its a tab,
  count it, and make the adjustments as necessary for eventual
  completions.

=cut

######################################################################
sub keypressed
{
  my ($this,$key,$actline) = @_;
  #$this->Debug("pressed key $key\n");
  if($key eq "^I")
  {
    #$this->Debug("hey its the tab key!!! need to complete $actline\n");
    if($this->{"countTAB"} == 0)
    {
      #first TAB call! save the completion
      #$this->Debug("first tab key!!!\n");
      $this->{"lastTAB"} = $actline;
    }# if($this->{"countTAB"} == 0)
    else
    {
      #restore the last completion to be able to cycle
      $actline = $this->{"lastTAB"};
      #$this->Debug("cycling tab key!!! resetting to $actline\n");
    }# else

    my $toreturn;
    my @available;
    my $preamble;

    if($actline =~/(.*?)\s+(.*)/)
    {
      $preamble = "$1 ";
      my $tocomplete = $2;
      #$this->Debug("completing argument $tocomplete\n");

      my $shCommands = $this->{"completions"};
      #$this->Debug("inserted arg completsions ".join(' ',@{$this->{"completions"}})." \n");
      @available = grep( ($_ =~ /^$tocomplete/i), @{$shCommands});
    }# if($actline =~/\s+/)
    else
    {
      #$this->Debug("completing command\n");
      my $shCommands = $this->{"shellCmds"};
      @available = grep( ($_ =~ /^$actline/i), keys(%$shCommands));

    }# else

    #$this->Debug("available completions".Dumper(\@available)." num=".scalar(@available)."\n");
    if(scalar(@available) == 0)
    {
      $this->Debug("no available completion\n");
      return undef;
    }# if(scalar(@available) == 0)
    my $actindex = $this->{"countTAB"} % scalar(@available);
    #$this->Debug("returnin $actindex th elmenet: ".$available[$actindex]."\n");
    $toreturn =  $preamble if($preamble);
    $toreturn .=  $available[$actindex];
    $toreturn =~ s/^I//g;
    $toreturn =~ s/	//g;

    $this->{"countTAB"} += 1;
    #$this->Debug("raised TAB count to ".$this->{"countTAB"}."\n");
    return $toreturn;
  }# if($key eq "^I")
  else
  {
    $this->{"countTAB"} = 0;
    delete $this->{"lastTAB"};
    #$this->Debug("resetted TAB count on key $key \n");
  }#else
  return "";
}# sub kypressed
######################################################################
=pod

=item setargCompletions 

getter to setup the argument completion list

=cut

######################################################################
sub setargCompletions
{
  my ($this,$listing) = @_;
  #$this->Debug("received arg listing ".Dumper($listing)." \n");

  #my @temp = split(/ /,$listing->{"listing"});
  my @temp = split(/ /,$listing);
  $this->{"completions"} = \@temp;
  #$this->Debug("inserted arg completsions ".join(' ',@{$this->{"completions"}})." \n");
}# sub setargCompletions
1
__END__


=back

=head1 THANKS

I. Wronsky came up with the concept for leverloda
This gave B. Boettcher the incentive to begin this work, the
famous itching paradigm!

=head1 SEE ALSO

http://www.geocities.com/iwronsky/
http://poe.perl.org/

=cut

