#!/usr/bin/perl
package POE::Component::goanseech::fileWatcher;
##############################################################################
=pod

=head1 NAME

fileWatcher

=head1 DESCRIPTION

scans directories and parses the files that are pointed inside if
they match something inside the patternServer.


=head1 AUTHOR

  Bruno Bttcher <bboett at adlp dot org>

=head1 COPYRIGHT and LICENCE

 Copyright (c) 2002 Bruno Boettcher

 fileWatcher.pm is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


=cut

##############################################################################
use warnings;
use strict;
use Data::Dumper;

use POE;
use POE::Filter::Reference;
use POE::Component::Client::TCP;
##############################################################################
=pod

=item CTOR

instantiation of the fileWatcher

=cut

##############################################################################
sub new 
{
  my($class,$sessionname) = @_;
  my $this = bless {}, "POE::Component::goanseech::fileWatcher";
  $this->{"name"} = $sessionname;
  return $this;
}
##############################################################################
=pod

=item _start

to keep this session alive set an alias

=cut
######################################################################
sub _start
{
  my ( $this,$kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  $_[KERNEL]->alias_set($this->{"name"});
  $kernel->yield("readDirs");  
  #$_[KERNEL]->post("scanFiles", "sendBatch" );
}# sub _start
##############################################################################
=pod

=item readDirs


=cut
######################################################################
sub readDirs
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  my $tmpFiles = $this->{"settings"}->{"rootdir"} . "files.tmp";
  open(FILE,">$tmpFiles");
  foreach my $directory (@{$this->{"dir_list"}})   
  {
    if(opendir(DIRLIST,$directory))
    {
      my @dirlisting = grep { /\.avi|\.asf|\.ogm/ && -f "$directory/$_" }readdir(DIRLIST);
      foreach my $filename (@dirlisting) 
      {
	#add only files we haven't allready checked
	if(!($this->{"allreadyParsed"} =~ /\Q$filename\E/i))
	{
	  push( @{$this->{"files"}},$filename) 
	}
	  #overwrite the file db with the actual list of present files
	print (FILE  "$filename\n");
      }# foreach my $filename (@dirlisting) 
      closedir(DIRLIST);
    }# if(opendir(DIRLIST,$directory))
  }# foreach my $directory (@{$this->{"dir_list"}}dd)   
  close (FILE);

  my @request = ();
  foreach my $afile (sort(@{$this->{"files"}}))
  {
    print("New file detected : $afile\n");
    push(@request, [ "find", 
	{ "line" =>$afile, "mode" => 1 }
	]);
  }

  #force a backup of the config files since changes were made
  push(@request, [ "save"]);

  #scedule the next round
  if($this->{"settings"}->{"dirscan"} >0)
  {
    print("posting next dirscan in ".$this->{"settings"}->{"dirscan"}."min\n");
    $kernel->delay("readDirs" => $this->{"settings"}->{"dirscan"}*60);
  }
  else
  {
    #one shot call
    push(@request, [ "quit"]);
  }# else

  if(scalar(@request <= 0))
  {
    print("Readirs calling doneBatch\n");
    $_[KERNEL]->yield('doneBatch');
    return;
  }# if(scalar(@request <= 0))
  #print("exiting readDirs with callback doneBatch@".$this->{"name"}."\n");
  my $job = {
    "requests" => \@request,
    "session" => $this->{"name"},
    "event"	=> "doneBatch"
  };
  $_[KERNEL]->post($this->{"querySession"}, $this->{"queryEvent"}, $job);
}# sub readDirs
##############################################################################
=pod

=item _stop

to keep this session alive set an alias

=cut
######################################################################
sub _stop
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  print("stopping files watcher...\n");
  #exit(0);
}# sub _stop
##############################################################################
=pod

=item shutdown

close this session

=cut
######################################################################
sub shutdown
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  #print("shutdown extract files...\n");
  $_[KERNEL]->post("pattern_client", "shutdown"); 
  #$kernel->yield("shutdown");
  #print("TCP client should now shutdown\n");
  $_[KERNEL]->alias_remove( "scanFiles" );
  #$_[KERNEL]->yield("shutdown" );
  #$_[KERNEL]->alias_remove( "scanFiles" );
  #$_[KERNEL]->alias_remove( "pattern_client" );
  undef $this->{"server"};
  exit();
}# sub shutdown

##############################################################################
=pod

=item init

Initiaize, means set up this module

=cut

######################################################################
sub init
{
  my ($this,$settings,$arguments) = @_;

  $this->{"settings"} = $settings;
#read in the urls to monitor
  $settings->{"basepath"} .= "." if(!($settings->{"basepath"}));
  $settings->{"basepath"} .= "/" if(!($settings->{"basepath"} =~ /\/$/));
  $settings->{"rootdir"} .= "/" if(!($settings->{"rootdir"} =~ /\/$/));



  foreach my $key (keys(%$arguments))
  {
    $this->{$key} = $arguments->{$key};
  }# foreach my $key (keys($arguments))

#start pages for this to search for
$this->{"dir_list"} = [];
# my @{$this->{"dir_list"}} = (
#      '/video/upload/bt',
#      '/video/upload/lopster',
#      '/video/upload/mldonkey',
#      '/video/upload/',
#      );

#read in the directories that should be monitored
my $directoryConf = $this->{"settings"}->{"rootdir"}."directories.conf";
if(open(FILE,$directoryConf))
{
  foreach my $line (<FILE>)
  {
    chomp($line);
    #add the line only if it isn't a comment
    push(@{$this->{"dir_list"}},$line) if($line && !($line =~ /^#/));
  }# foreach my $line (<FILE>)
  close(FILE);
}# if(open(FILE,"directories.conf"))
else
{
  print("couldn't open directories.conf\n");
}# else



#read in a temp file where we store the files we allready found and checked
#out...

 $this->{"allreadyParsed"} = "";
my $tmpFiles = $this->{"settings"}->{"rootdir"}."files.tmp";
if(open(FILE,$tmpFiles))
{
  foreach my $line (<FILE>)
  {
    chomp($line);
    $this->{"allreadyParsed"} .= " $line";
  }# foreach my $line (<FILE>)
  close(FILE);
}# if(open(FILE,"directories.conf"))
#print("Files we allready parsed:  $this->{"allreadyParsed"}\n") if ( $this->{"allreadyParsed"});

$this->{"jobcount"} = 0;

#fetch the stuff from the file
$this->{"files"} = []; 
$this->{"matches"} = {}; 

  $this->startSession();

}#sub init
##############################################################################
=pod

=item startSession

start our own session to be able to have timered stuff

=cut
######################################################################
sub startSession
{
  my ($this) = @_;


  POE::Session->create
    (
     object_states =>
     [      

     $this => { 
     _start         => "_start",
     _stop         => "_stop",
     "shutdown" => "shutdown",
     "readDirs" => "readDirs",
     "doneBatch" => "doneBatch",
     }
  ]
    );
}# sub startSession
##############################################################################
=pod

=item doneBatch

The series server finished parsing a batch of lines, if 

=cut
######################################################################
sub doneBatch
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  #print("Filewatcher entering doneBatch\n");

  foreach my $hash (@$arg)
  {
    print("donwBatch processing ".Dumper($hash)."\n");
    print("DW: new ".$hash->{"pattern"}." for ".$hash->{"line"}."\n");
  }#// foreach my $hash ($arg)

}# sub doneBatch
1
__END__


=back

=head1 SEE ALSO

http://poe.perl.org/

=cut

