#!/usr/bin/perl
package POE::Component::goanseech::torrentCollector;
##############################################################################
=pod

=head1 NAME

POE::Component::goanseech::torrentCollector

=head1 DESCRIPTION

extracts the links from a given HTML file and
recusrively parses the files that are pointed inside 

=head1 AUTHOR

  Bruno Bttcher <bboett at adlp dot org>

=head1 COPYRIGHT and LICENCE

 Copyright (c) 2003 Bruno Boettcher

 POE::Component::goanseech::torrentCollector is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


=cut

##############################################################################
use warnings;
use strict;
use Data::Dumper;

use POE;
use POE::Wheel::Run;
use File::Copy;

use POSIX qw(strftime);



##############################################################################
=pod

=item tc_stop

to keep this session alive set an alias

=cut
######################################################################
sub tc_stop
{
  #print("stopping extract torrents...\n");
  #exit(0);
}# sub tc_stop
##############################################################################
=pod

=item shutdown

close this session

=cut
######################################################################
sub shutdown
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  #print("shutdown extract torrents...\n");
  $_[KERNEL]->alias_remove( $this->{"name"});
  exit();
}# sub shutdown
##############################################################################
=pod

=item CTOR

instantiation of the Helper

=cut

##############################################################################
sub new 
{
  my($class,$sessionname) = @_;
  my $this = bless {}, "POE::Component::goanseech::torrentCollector";
  $this->{"name"} = $sessionname;
  return $this;
}
##############################################################################
=pod

=item init

Initiaize, means set up this module

=cut

######################################################################
sub init
{
  my ($this,$settings,$arguments) = @_;

  $this->{"settings"} = $settings;
  $this->{"requested"} = {};
  $this->{"stack"} = [];
  $this->{"kill"} = [];
  $this->{"semaphore"} = 0;
  $this->{"downloading"} = " ";

  foreach my $key (keys(%$arguments))
  {
    $this->{$key} = $arguments->{$key};
  }# foreach my $key (keys($arguments))

  $this->startSession();
}#sub init
##############################################################################
=pod

=item startSession

start our own session to be able to have timered stuff

=cut
######################################################################
sub startSession
{
  my ($this) = @_;

  POE::Session->create
    (
     object_states =>
     [      

     $this => { 
     _start         => "tc_start",
     _stop         => "tc_stop",
     "shutdown" => "shutdown",
     "gotTorrents" => "gotTorrents",
     "chebang" => "parse_start",
     "process" => "fetchTorrent",
     "bt_result"  => "bt_result",
     "bt_debug"  => "bt_debug",
     "bt_done"  => "bt_done",
     "killit"  => "killit",
     "dummy"  => "dummy",
     }
  ]
    );
}# sub startSession
##############################################################################
=pod

=item tc_start

to keep this session alive set an alias

=cut
######################################################################
sub tc_start
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  $_[KERNEL]->alias_set($this->{"name"});
  #start the processing....
  my $handler =  $this->{"urlHandler"};
  $handler->{"indexRun"} = 1;
  my @tmp = @{$handler->{"index_list"}};
  $this->{"index_list"} = \@tmp;
  $this->{"kernel"} = $kernel;

  $kernel->yield("chebang");
}# sub tc_start
##############################################################################
=pod

=item gotTorrents

we have some torrents to check out

=cut
######################################################################
sub gotTorrents
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  #$kernel->post( $this->{"urlSession"}, $this->{"urlEvent"});
  #$this->print("storing torrents into ".$this->{"settings"}->{"batchfile"}."\n");
  #if(open(FILE,">>".$this->{"settings"}->{"batchfile"}) )
  #{

      my $msg2 = "START of gotTorrents!\n ";
    my $msg;
    foreach my $line (@{$arg})
    {
      my $urlref = $line->{"pattern"};
      $msg2 .= "checking for duplicate of $urlref ";
      if(!($this->{"requested"}->{$urlref}))
      {
	$msg2 .= "not a duplicate ";
	$this->{"requested"}->{$urlref} = $line;
	push(@{$this->{"stack"}},$line);
	$this->print("new incoming urlref: ".Dumper($line)."\n");
      }# if(!($this->{"requested"}->{$urlref}))
      else
      {
	$msg2 .= "duplicate\n ";
	$this->print("Line allready processed: ".Dumper($this->{"requested"}->{$urlref})."\n");
      }# else
      #$msg .=  "btdownloadheadless --max_uploads 1 --url ";
      #$msg .= $arg->{$urlref}."\n";
    }# foreach my $pattern (keys(%$torrent))

    my $semaphore = $this->{"semaphore"};
    if($this->{"settings"}->{"concurrentTorrents"} < 1)
    {
      die("you should set concurrentTorrents at least to 1 in your settings file!!\n");
    }# if($this->{"settings"}->{"concurrentTorrents"} < 1)
    my $maxconcurrent = $this->{"settings"}->{"concurrentTorrents"};
    while($this->{"semaphore"} < $maxconcurrent && scalar(@{$this->{"stack"}}) >0)
    {
      print("casting off new torrent since ".$this->{"semaphore"}." of $maxconcurrent....\n");
	$msg2 .= " torrent instances free and urls to process\n ";
      #$this->debug($msg2);
      # not enough  torrent requestis running and we have outstanding jobs...
      $this->{"semaphore"} += 1;
      $kernel->yield("process");
    }# if($this->{"semaphore"} && scalar(@{$this->{"stack"}}) >0)
    #else
    #{
      $this->print("still running \n") if($semaphore >= $maxconcurrent); 
      $this->print("stack empty \n") if(scalar(@{$this->{"stack"}}) <=0); 
	#$msg2 .= " no torrent instances free or no urls to process\n ";
      $this->debug($msg2);
    #}# else

    # we are done kick in the next round
    #  $_[KERNEL]->delay(alive => 5);
    my $next_time = time+$this->{"settings"}->{"frequence"}*60;
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = localtime($next_time);
    $year += 1900;
    $mon++;
    my $now_string .= "going to sleep for "; 
    $now_string .= $this->{"settings"}->{"frequence"}."min wake up at ";
    $now_string .= "$mday/$mon/$year $hour:$min:$sec UTC or ";

    $now_string .= strftime("%H:%M:%S", localtime($next_time));

    $now_string .= "\n";


    $this->print($now_string);
    $kernel->delay("chebang" => $this->{"settings"}->{"frequence"}*60);
    #if($msg)
    #{
      #  $msg =~ s/\Q'\E/\Q\'\E/g;
      #  $msg =~ s/\Q&\E/\Q\&\E/g;
      #  $msg =~ s/\Q(\E/\Q\(\E/g;
	    #        $msg =~ s/\Q)\E/\Q\)\E/g;
      #  print FILE $msg;
      #}# if($msg)
    #close(FILE);
    #}# if(open(">>seeds") )
    #print("all stuff processed exiting\n");
    #$kernel->post($this->{"name"}, "shutdown"); 
}# sub gotTorrents
##############################################################################
=pod

=item parse_start

kick all this stuff into motion

=cut
######################################################################
sub parse_start
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  #start the processing....
  my @tmp = @{$this->{"index_list"}};

  $kernel->post( $this->{"urlSession"}, "reset",\@tmp);
  $kernel->post( $this->{"urlSession"}, $this->{"urlEvent"},\@tmp);
}# sub parse_start
##############################################################################
=pod

=item fetchTorrent

we seem to have some work to do.... cast of the forks with the torrent retrievers...

=cut
######################################################################
sub fetchTorrent
{
  my ($this, $kernel, $arg ) = @_[OBJECT, KERNEL, ARG0 ];
  #start the processing....
  my @tmp = @{$this->{"index_list"}};
  $this->print( "fetchtorrent\n" );
	my $msg2 = " FETCHTORRENT\n ";

  #if(open(FILE,">>".$this->{"settings"}->{"batchfile"}) )
  #{

    my $msg;
    my $line = shift(@{$this->{"stack"}});
    my $entry = $line->{"url"};
    if($entry && !($entry =~ /^\s*$/))
    {
      $this->print( "entry in stack\n" );
      $entry = $this->quotemeta($entry);
      my $pattern = $line->{"pattern"};
	$msg2 .= " cmp $pattern with ".$this->{"downloading"}."\n ";
      if(!($this->{"downloading"} =~ /\Q$pattern\E/))
      {
	$msg2 .= " no trace of $pattern downloading\n ";
	$this->{"downloading"} .= $pattern." ";
	#$msg =  "btdownloadheadless --max_uploads 1 --url $entry\n";
        $msg = $this->{"settings"}->{"torrentcmd"};
	$msg .=  " --url $entry\n";
	$this->print("about to issue system call: $msg\n");
	my $task = POE::Wheel::Run->new(
	    Program => $msg,
	    StdioFilter  => POE::Filter::Line->new(),    # Child speaks in lines.
	    StderrFilter => POE::Filter::Line->new(),
	    StdoutEvent  => "bt_result",
	    StderrEvent  => "bt_debug",
	    CloseEvent   => "bt_done",
	    );


	    $this->{"tasks"}->{$task->ID} = { 
	      "process" => $task, "file" => $entry, "percent" => 0} ;
      }# if(!($this->{"downloading"} =~ /\Q$entry\E/))
      else
      {
	$this->print("seems we are allready fetching $entry\n");
	$msg2 .= " found trace of $entry downloading\n ";

      }# else

    }# foreach my $pattern (keys(%$torrent))
    else
    {
	$this->print( "stack empty\n" );
        $this->{"semaphore"} -= 1 if($this->{"semaphore"} >0);
    }
      $this->debug($msg2);

    #if($msg)
    #{
      #  print FILE $msg;
      #}# if($msg)
    #close(FILE);
    #}# if(open(">>seeds") )
}# sub fetchTorrent

######################################################################
=pod

=item bt_result

Handle information returned from the task.  Since we're using
POE::Filter::Reference, the $result is however it was created in the
child process.  its a hash reference following the format requested in DB::Helper

=cut

######################################################################
sub bt_result 
{
  my ($this,$kernel,$result,$wid) = @_[OBJECT,KERNEL,ARG0,ARG1];
  #$result = quotemeta($result);
  #print("Result[$wid] $result\n");
  return if($this->{"tasks"}->{$wid}->{"completed"});

  if($result =~ /^percent.*?done:.*?(\d+)\.(\d+)/)
  {
    my ($completion,$frac) = ($1,$2);

    

    my $msg = "BT: ";
    if($this->{"tasks"}->{$wid}->{"realfile"})
    {
      $msg = "BT: ".$this->{"tasks"}->{$wid}->{"realfile"};
    }
    else
    {
      $msg = "BT: ".$this->{"tasks"}->{$wid}->{"file"};
    }
    #the first line gives us a percentage of 100....
    if($completion == 100 && $this->{"tasks"}->{$wid}->{"percent"} &&
	!($this->{"tasks"}->{$wid}->{"checking"}))
    {
      $msg .= " COMPLETE";
      push(@{$this->{"kill"}}, $this->{"tasks"}->{$wid});
      my $tmout = $this->{"settings"}->{"uploaddelay"};

      $msg .= "\nkill in ${tmout}min\n";
      $tmout *= 60;
      $kernel->delay("killit" => $tmout);
      $this->{"tasks"}->{$wid}->{"completed"} = 1;

      #first try to move the file....
      my $filename = $this->{"tasks"}->{$wid}->{"file"};
      my $dir = $this->{"settings"}->{"uploaddir"};
      $dir .= '/' if(!($dir =~ /\/$/));
      my  $target = $dir.$filename;

      $this->print("try1 to move $filename to $target\n");
      move($filename,$target);

      $filename = $this->{"tasks"}->{$wid}->{"realfile"};
      if($filename)
      {
        # $filename =~ s/\Q'\E/*/g;
        # $filename =~ s/\Q&\E/*/g;
        # $filename =~ s/\Q(\E/*/g;
        # $filename =~ s/\Q)\E/*/g;
        # $filename =~ s/!/*/g;
        # 
        # $target = $dir.$filename;
        $this->print("try1b to move $filename to $target\n");
	move($filename,$target);
      }# if($filename)
    }# if($1 == 100)
    else
    {
      $msg .= ' %:'.$completion.".".$frac;
    }# else
    $this->{"tasks"}->{$wid}->{"percent"} = $completion;
    $this->{"tasks"}->{$wid}->{"msg"} = $msg;
  }# if($result =~ /^percent.*?done:.*?(\d+)\.(\d+)/)
  elsif($result =~ /Download Failed/i)
  {
    $this->print("Failure in trying to fetch ".$this->{"tasks"}->{$wid}->{"file"}."\n");
    
    if(open(FILE,">>".$this->{"settings"}->{"errorfile"}) )
    {
      print FILE "FAILURE: ".$this->{"tasks"}->{$wid}->{"file"}."\n";
      $this->{"tasks"}->{$wid}->{"error"} = 1;
      close(FILE);
    }#if(open(FILE,">>".$this->{"settings"}->{"errorfile"}) )
  }# elsif($result =~ /^saving:\s*(.*?)\s+\(\d+.*?MB\)/)
  elsif($result =~ /^saving:\s*(.*?)\s+\(\d+.*?MB\)/)
  {
    $this->{"tasks"}->{$wid}->{"realfile"} = $1;
    #print("file: $1\n");
  }# elsif($result =~ /^saving:\s*(.*?)\s+\(\d+.*?MB\)/)
  elsif($result =~ /^download\s+rate:\s+(.*)/i)
  {
    $this->{"tasks"}->{$wid}->{"msg"} .= " D:$1";
  }#elsif($result =~ /^download rate:\s*(.*?)/)
  elsif($result =~ /^upload\s+rate:\s+(.*)/i)
  {
    $this->{"tasks"}->{$wid}->{"msg"} .= " U:$1\n";
    $this->print($this->{"tasks"}->{$wid}->{"msg"});
    delete $this->{"tasks"}->{$wid}->{"msg"};
  }#elsif($result =~ /^upload rate:\s*(.*?)/)
  elsif($result =~ /^time left:\s*(.*)/i)
  {
    my $time = $1;
    if($time =~ /checking/i)
    {
      $this->{"tasks"}->{$wid}->{"checking"} = 1;
    }# if($time =~ /checking/i)
    else
    {
      delete $this->{"tasks"}->{$wid}->{"checking"};
    }# else

    $this->{"tasks"}->{$wid}->{"msg"} .= " ETA: $time";
    if($time =~ /Download Succeeded/i )
    {
      push(@{$this->{"kill"}}, $this->{"tasks"}->{$wid});
      my $tmout = $this->{"settings"}->{"uploaddelay"};

      my $msg .= "\nkill in ${tmout}min";
      $tmout *= 60;
      $kernel->delay("killit" => $tmout);
      $this->{"tasks"}->{$wid}->{"completed"} = 1;
      $this->print($this->{"tasks"}->{$wid}->{"msg"}.$msg);
    }# if($1 == 100)
  }#elsif($result =~ /^time left:\s*(.*?)/)
}# sub bt_result 
######################################################################
=pod

=item bt_debug

Catch and display information from the child's STDERR.  This was
useful for debugging since the child's warnings and errors were not
being displayed otherwise.

=cut

######################################################################
sub bt_debug 
{
  my ($this,$result) = @_[OBJECT,ARG0];
  $result = $this->quotemeta($result);
  $this->print("Debug: $result\n");
}
######################################################################
=pod

=item bt_done

A job completed, push the returning helper onto the stack of available workers, check for waiting requests and eventuall fire off an doQuery or batch event to start their processing.

=cut

######################################################################
sub bt_done 
{
  my ($this, $kernel, $heap, $task_id ) = @_[OBJECT, KERNEL, HEAP, ARG0 ];
  $this->print("job done returning from a fork for id $task_id\n");
  #bail out if this ended in an error
  if(!($this->{"tasks"}->{$task_id}->{"error"}))
  {
    my $filename = $this->{"tasks"}->{$task_id}->{"file"};
    my $urlref = $this->{"tasks"}->{$task_id}->{"url"};
    delete $this->{"requested"}->{$urlref} if($this->{"requested"} && $this->{"requested"}->{$urlref});

    my $entry = $this->quotemeta($filename);
    my @patterns = split(/ /,$this->{"downloading"});
    foreach my $pattern (@patterns)
    {
      if($entry =~ /$pattern/)
      {
        $this->{"downloading"} =~ s/\Q$pattern\E//g;
        $this->{"downloading"} =~ s/\s+/ /g;
	last;
      }# if($entry =~ /$pattern/)
    }# foreach my $pattern (@patterns)

    my $dir = $this->{"settings"}->{"uploaddir"};
    $dir .= '/' if(!($dir =~ /\/$/));
    my  $target = $dir.$filename;

    $this->print("try2 to move $filename to $target\n");
    move($filename,$target);
    #let the series advance

    my @request = ( 
	[ "find", { "line" =>$filename, "mode" => 1} ]
	);
    #reset the listing of pages to seach, since we are asking for a whole new bunch of pages...
    my $job = {
      "requests" => \@request,
      "session" => $this->{"name"},
      "event"	=> "dummy",
	"tmout" => time(),
    };
    $_[KERNEL]->post($this->{"querySession"}, $this->{"queryEvent"}, $job);




    $filename = $this->{"tasks"}->{$task_id}->{"realfile"};
    if($filename)
    {

      #$filename =~ s/\Q'\E/\Q\'\E/g;
      #$filename =~ s/\Q&\E/\Q\&\E/g;
      #$filename =~ s/\Q(\E/\Q\(\E/g;
	  #	  $filename =~ s/\Q)\E/\Q\)\E/g;
      $target = $dir.$filename;
      $this->print("try2b to move $filename to $target\n");
      move($filename,$target);

      my @request = ( 
	  [ "find", { "line" =>$filename, "mode" => 1} ]
	  );
      #reset the listing of pages to seach, since we are asking for a whole new bunch of pages...
      my $job = {
	"requests" => \@request,
	"session" => $this->{"name"},
	"event"	=> "dummy"
      };
      $_[KERNEL]->post($this->{"querySession"}, $this->{"queryEvent"}, $job);
    }# if($filename)
  }

  delete $this->{"tasks"}->{$task_id};
  $this->{"semaphore"} -= 1 if($this->{"semaphore"} > 0);

  my $semaphore = $this->{"semaphore"};
  my $maxconcurrent = $this->{"settings"}->{"concurrentTorrents"};
  while($this->{"semaphore"} < $maxconcurrent && scalar(@{$this->{"stack"}}) >0)
  {
    # not enough  torrent requestis running and we have outstanding jobs...
    $this->print( "posting process!! from bt_done\n" );
    $this->{"semaphore"} += 1;
    $kernel->yield("process");
  }# if($this->{"semaphore"} && scalar(@{$this->{"stack"}}) >0)
  #else
  #{
    $this->print("still enough running \n") if($semaphore >= $maxconcurrent); 
    $this->print("stack empty \n") if(scalar(@{$this->{"stack"}}) <=0); 
  #}# else
}# sub bt_done 
######################################################################
=pod

=item killit

A transfer was marked as completed, kill it and move the file to the
receiving area

=cut

######################################################################
sub killit 
{
  my ($this, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
  #print("kill job job done returning from a fork :".Dumper($this->{"kill"})."\n");
  $this->print("kill job job done returning from a fork\n");

  my $entry;
  while ($entry = shift(@{$this->{"kill"}}))
  {
    $entry->{"process"}->kill();
  }
}# sub killit 
######################################################################
=pod

=item quotemeta

need to passificate the incoming file string but not all of it...

=cut

######################################################################
sub quotemeta 
{
  my ($this, $entry) = @_;
  #print("quoting $entry\n");
  #$entry =~ s/^\s*(.*?)\s*$/$1/;
  $entry = quotemeta($entry);
  #return sme of the stuff to original
  $entry =~ s/http\\:/http:/g;
  $entry =~ s/\\\//\//g;
  $entry =~ s/\\\././g;
  $entry =~ s/\\\?/?/g;
  $entry =~ s/\\=/=/g;
  $entry =~ s/\\-/-/g;
  $entry =~ s/\\\[/[/g;
  $entry =~ s/\\\]/]/g;
  $entry =~ s/\\ /%20/g;
  $entry =~ s/\\~/~/g;
  $entry =~ s/\\%/%/g;
  #$entry =~ s/\Q'\E/\Q\'\E/g;
  #$entry =~ s/\Q&\E/\Q\&\E/g;
  #$entry =~ s/\Q(\E/\Q\(\E/g;
	#$entry =~ s/\Q)\E/\Q\)\E/g;
  #$entry =~ s/ /\ /g;
  return $entry;
}# sub quotemeta 
######################################################################
=pod

=item dummy

dummy event to collect return events that we aren't interested in

=cut

######################################################################
sub dummy 
{
  my ($this, $kernel, $heap,$arg) = @_[OBJECT, KERNEL, HEAP,ARG0];
  #print("kill job job done returning from a fork :".Dumper($this->{"kill"})."\n");
  $this->print("Dummy: ".Dumper($arg)."\n");
}# sub dummy 
#########################################################
=pod

=item debug

issue debugging messages to a file

=cut

#########################################################
sub debug
{
  my ($this,$msg) = @_;
  $msg = $this if(!$msg);
  if(open(DEBUG,">>debugging"))
  {
      print( DEBUG "$msg\n");
      close(DEBUG);
  }
  else
  {
      print("$msg\n");
  }
}# sub debug
#########################################################
=pod

=item print

delegator for printing

=cut

#########################################################
sub print
{
  my ($this,$msg) = @_;

  if($this->{"kernel"})
  {
    $this->{"kernel"}->post($this->{"querySession"}, "print", $msg,"torrent") 
  }
  else
  {
    print($msg);
  }# else
}# sub print
1
__END__

=back


=head1 SEE ALSO

http://poe.perl.org/

=cut

